// generated by CommonLibrary.cmake, do not edit.

/**
 * @file include/co/version.h
 * Defines version macros and class for library Collage.
 */

#ifndef COLLAGE_VERSION_H
#define COLLAGE_VERSION_H

#include <co/api.h>
#include <string>

namespace co
{
    /** The current major version. */
#   define CO_VERSION_MAJOR 1

    /** The current minor version. */
#   define CO_VERSION_MINOR 7

    /** The current patch level. */
#   define CO_VERSION_PATCH 0

    /** The current SCM revision. */
#   define CO_VERSION_REVISION 0x0

    /** The current binary interface. */
#   define CO_VERSION_ABI 7ull

    /** The current major.minor.version as a string. */
#   define CO_VERSION_STRING \
    "1.7.0"

    /** The current major.minor.version.revision as a string. */
#   define CO_REV_STRING \
    "1.7.0.0x0"

/** True if the current version is newer than the given one. */
#   define CO_VERSION_GT( MAJOR, MINOR, PATCH )       \
    ( (CO_VERSION_MAJOR>MAJOR) ||                     \
      (CO_VERSION_MAJOR==MAJOR && (CO_VERSION_MINOR>MINOR || \
          (CO_VERSION_MINOR==MINOR && CO_VERSION_PATCH>PATCH))))

/** True if the current version is equal or newer to the given. */
#   define CO_VERSION_GE( MAJOR, MINOR, PATCH )       \
    ( (CO_VERSION_MAJOR>MAJOR) ||                     \
      (CO_VERSION_MAJOR==MAJOR && (CO_VERSION_MINOR>MINOR || \
          (CO_VERSION_MINOR==MINOR && CO_VERSION_PATCH>=PATCH))))

/** True if the current version is older than the given one. */
#   define CO_VERSION_LT( MAJOR, MINOR, PATCH )       \
    ( (CO_VERSION_MAJOR<MAJOR) ||                     \
      (CO_VERSION_MAJOR==MAJOR && (CO_VERSION_MINOR<MINOR || \
          (CO_VERSION_MINOR==MINOR && CO_VERSION_PATCH<PATCH))))

/** True if the current version is older or equal to the given. */
#   define CO_VERSION_LE( MAJOR, MINOR, PATCH )       \
    ( (CO_VERSION_MAJOR<MAJOR) ||                     \
      (CO_VERSION_MAJOR==MAJOR && (CO_VERSION_MINOR<MINOR || \
        (CO_VERSION_MINOR==MINOR && CO_VERSION_PATCH<=PATCH))))

/** Information about the current Collage version. */
class CO_API Version
{
public:
    /** @return the current major version of Collage. */
    static int getMajor();

    /** @return the current minor version of Collage. */
    static int getMinor();

    /** @return the current patch level of Collage. */
    static int getPatch();

    /** @return the current Collage version (MM.mm.pp). */
    static std::string getString();

    /** @return the SCM revision. */
    static unsigned long long getRevision();

    /** @return the current binary interface version of Collage. */
    static int getABI();

    /** @return the current Collage version plus the git SHA hash (MM.mm.pp.rev). */
    static std::string getRevString();

    /**
     * Runtime check for ABI compatibility.
     *
     * Call from code using Collage. Will fail if the executable
     * was compiled against a version incompatible with the runtime version.
     *
     * @return true if the link-time and compile-time DSO are compatible.
     */
    static bool check()
    {
        return getMajor()==CO_VERSION_MAJOR &&
               getMinor()==CO_VERSION_MINOR;
    }

    /** @return JSON schema describing this object. */
    static std::string getSchema()
    {
        return R"(
                {
                  "title": "Version",
                  "type": "object",
                  "description": "Version information of the running application",
                  "properties": {
                    "major": { "type": "integer" },
                    "minor": { "type": "integer" },
                    "patch": { "type": "integer" },
                    "abi": { "type": "integer" },
                    "revision": { "type": "string" }
                  }
               }
               )";
    }

    /** @return JSON-encoded data for this object. */
    static std::string toJSON()
    {
        return R"(
               {
                 "major": 1,
                 "minor": 7,
                 "patch": 0,
                 "abi": 7,
                 "revision": "0"
               }
               )";
    }
};

}

#endif
